import TextClassification.classification_metrics as cls_metrics
import glob
import sys, os
from TextClassification.argsparse_classification_preamble import argsparse_preamble
import pickle
import database_preparation.utils_labeled_datasets as dt
sys.path.append(os.getcwd())

# script parameters:
metrics_folder = "cls_metrics/metrics_new"


def generate_save_conf_matrix(model_name, clustered_data,
                              test_set_index):
    ob_dir = "./TextClassification/" + metrics_folder + "/" \
             + clustered_data + "_clustered_" + model_name + "_classified.pickle"

    try:
        with open(ob_dir, 'rb') as f:
            metrics = pickle.load(f)
    except FileNotFoundError:
        return False

    # plot confusion matrix
    if "ger-patho-bert" in model_name:
        titlename = "Patho-BERT"
    elif "german" in model_name:
        titlename = "German-BERT"
    elif "SGD" in model_name:
        titlename = "SGD-classifier"
    elif "MLP" in model_name:
        titlename = "MLP-classifier"
    else:
        titlename = model_name
    unique_labels = dt.get_amount_unique_labels(clustered_data)
    labels = [a for a in range(unique_labels)]
    # labels=['class'+str(a) for a in range(unique_labels)]
    # https://matplotlib.org/3.5.1/tutorials/colors/colormaps.html
    metrics.plot_confusion_matrix(labels, prediction_set=test_set_index,
                                  plot=False, save=True,
                                  filename="confmatrx_"+clustered_data + "_clustered_" + model_name + "_classified",
                                  title= titlename, normalized=True, annot = False, colormap='gist_heat')
    return True

def print_f1_per_clusters(model_name, clustered_data):
    # print f1-scores for each class of a single test run
    ob_dir = "./TextClassification/" + metrics_folder + "/" \
             + clustered_data + "_clustered_" + model_name + "_classified.pickle"
    try:
        with open(ob_dir, 'rb') as f:
            metrics = pickle.load(f)
    except FileNotFoundError:
        return False
    print("================ model: " + model_name + " | cluster-set: " + clustered_data + " ================")
    df = metrics.classes_scores(-1)
    print(df.to_latex().replace('{}', 'cluster'))

    return True

def main():
    args = argsparse_preamble()

    model_names = ['SGDClassifier', 'MLPClassifier', 'ger-patho-bert-2', 'bert-base-german-cased',
                   'CNN', 'RNN', 'LogisticRegression', 'MultinomialNB']

    # print f1 scores for each classification model:
    for model in model_names:
        print_f1_per_clusters(model, args.clustered_data)

    # print classification overview tables for each clustering method:
    print()
    file_list = glob.glob("./TextClassification/" + metrics_folder + "/" + '/*.json')
    for file in file_list:
        cls_metrics.print_results_as_latextable(file, True)
        print()

    # generate all confusion matrices for each classification model:
    for model in model_names:
        try:
            generate_save_conf_matrix(model, args.clustered_data, -1)
        except:
            print(f"cant generate conf matrix for {model}")


if __name__ == '__main__':
    main()