
#%% argsparse section
import sys, os

sys.path.append(os.getcwd())

from TextClustering.argsparse_clustering_preamble import argsparse_preamble
args = argsparse_preamble()

from database_preparation.utils_labeled_datasets import is_text_lst_tokenized

#%% import section
import pickle
from TextClustering.basedOn_BOW.GSDPMM import *
import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
import matplotlib.pyplot as plt
import umap
from tqdm import tqdm
from TextClustering.utils_metrics import ClusterMetrics
from database_preparation.preprocess import print_meta_data
from TextClassification.classification_for_cluster_evaluation import cross_validate_label_corpus_with_simple_SVM

#%% load the data
with open(args.path2corpus, 'rb') as f:
    diag_lst = pickle.load(f)

print_meta_data(args.path2corpus)

#%% and save it for DPMM
text = ['text'] * len(diag_lst)
if is_text_lst_tokenized(args.path2corpus):
    text_tupls = list(zip(text, [' '.join(tokenized_text) for tokenized_text in diag_lst]))
else:
    text_tupls = list(zip(text, [text for text in diag_lst]))
df = pd.DataFrame(text_tupls)
df.to_csv('TextClustering/basedOn_BOW/temp.csv', header=None)

def identity(word):
    return word

def create_vectorizer(data):
    vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
    return vec.fit_transform(data)

text_features = create_vectorizer(diag_lst)

#%% find the best hyperparameter
if args.find_k_value:

    # %% set the parameter
    args.alpha = 0.3
    args.beta = 0.02
    args.iterNum = 5
    args.dataset = 'TextClustering/basedOn_BOW/temp.csv'

    beta_list = np.arange(3,23,1)
    s_score, n_cluster, svm_scores = [], [], []
    n_steps = []
    for i in tqdm(beta_list):

        #%% initialize it
        gsdmm = GSDPMM(i,
                       args.alpha, args.beta,
                       args.iterNum,
                       args.dataset)
        gsdmm.initialize()

        # %% actually do it
        gsdmm.gibbs_sampling()

        #%% evalute the model
        evaluation = ClusterMetrics(text_features, gsdmm.z_c)
        s_score.append(evaluation.s_score)
        svm_scores.append(
            cross_validate_label_corpus_with_simple_SVM(gsdmm.z_c, args.path2corpus + '.pkl',
                                                        False))
        n_cluster.append(len(np.unique(gsdmm.z_c)))
        n_steps.append(i)



    #%% plot it
    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax3 = ax1.twinx()
    ax1.plot(n_steps, s_score, 'bx-')
    ax2.plot(n_steps, n_cluster, 'rx-')
    ax3.plot(n_steps, svm_scores, 'gx-')
    ax1.set_xlabel('Minimal cluster size')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('s-score')
    ax2.yaxis.label.set_color('red')
    ax2.set_ylabel('Number of clusters')
    ax3.yaxis.label.set_color('green')
    ax3.set_ylabel('svm accuracy')
    plt.title('Ellbow-method-like plot')
    plt.savefig("TextClustering/plots/elbow_method/GSDPMM_elbow_plot.png", dpi=300)
    plt.show()
    sys.exit()

#%% set the parameter
args.alpha = 0.3
args.beta = 0.02
args.iterNum = 5
args.dataset = 'TextClustering/basedOn_BOW/temp.csv'

gsdmm=GSDPMM(args.k_value,
             args.alpha, args.beta,
             args.iterNum,
             args.dataset)
gsdmm.initialize()

#%% actually do it
gsdmm.gibbs_sampling()

#%% retrieve the results
A=gsdmm.z_c
num_list=gsdmm.num_list
m_z=gsdmm.m_z
n_z=gsdmm.n_z
n_zv=gsdmm.n_zv
docu_num=gsdmm.docu_num
predictedCluster=gsdmm.z_c
wordid_array=gsdmm.wordid_array
wordfreq_array=gsdmm.wordfreq_array

#%% save UMAP data points:
umap_text_features2D = umap.UMAP(n_neighbors=15,
                                 n_components=2,
                                 min_dist=0.0, metric='cosine').fit_transform(text_features)
# save umaped vectors and labels:
df = pd.read_pickle(args.df_cases_file)
df['umapX_GSDPMM'] = umap_text_features2D[:, 0]
df['umapY_GSDPMM'] = umap_text_features2D[:, 1]
df['label_GSDPMM'] = predictedCluster
df.to_pickle(args.df_cases_file)

#%% evalute the model
from TextClustering.utils_metrics import ClusterMetrics
evaluation = ClusterMetrics(text_features, predictedCluster,
                            file_name= "TextClustering/cluster_metrics/GSDPMM_metrics.pkl")

evaluation.write_to_file()


