# %% import section
import pickle
from sklearn.feature_extraction.text import TfidfVectorizer
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
import pandas as pd
from database_preparation.preprocess import print_meta_data
from TextClassification.classification_for_cluster_evaluation import cross_validate_label_corpus_with_simple_SVM
import umap
from database_preparation.utils_stringpreparation import get_most_frequent_words
import numpy as np
from database_preparation.utils_labeled_datasets import is_text_lst_tokenized, is_text_lst_tfidf_vectorized
from TextClustering.argsparse_clustering_preamble import argsparse_preamble
import os

args = argsparse_preamble()

plot_real_diagnosis = False

def identity(word):
    return word

# %% load the data
with open(args.path2corpus, 'rb') as f:
    diag_lst = pickle.load(f)

text_is_vectorized = is_text_lst_tfidf_vectorized(args.path2corpus)
if not (is_text_lst_tokenized(args.path2corpus) or text_is_vectorized):
    print("Error: " + args.path2corpus + ' has to be a list of tokenized texts or tfidf-vectorized texts!')
    exit(1)

print_meta_data(args.path2corpus)

def create_vectorizer(data):
    vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
    vec = vec.fit_transform(data)
    return vec


if text_is_vectorized:
    text_features = diag_lst
else:
    text_features = create_vectorizer(diag_lst)


# %% perform elbow-method to find good cluster number
if args.find_k_value:
    Sum_of_squared_distances, svm_values = [], []
    K = range(2, 23, 1)
    for k in K:
        print("iteration #" + str(k))
        km = KMeans(n_clusters=k, max_iter=200, n_init=10)
        km = km.fit(text_features)
        predictedCluster_text_features = km.predict(text_features)
        Sum_of_squared_distances.append(km.inertia_)
        svm_values.append(
            cross_validate_label_corpus_with_simple_SVM(predictedCluster_text_features, args.path2corpus, False))

    fig, ax1 = plt.subplots()
    #ax2 = ax1.twinx()
    ax3 = ax1.twinx()
    ax1.plot(K, Sum_of_squared_distances, 'bx-')
    #ax2.plot(K, svm_values, 'rx-')
    ax3.plot(K, svm_values, 'gx-')
    ax1.set_xlabel('K')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('Sum_of_squared_distances')
    #ax2.yaxis.label.set_color('red')
    #ax2.set_ylabel('Number of clusters')
    ax3.yaxis.label.set_color('green')
    ax3.set_ylabel('svm accuracy')
    plt.title('Ellbow-method-like plot')
    plt.savefig("TextClustering/plots/elbow_method/KMeans_elbow_plot.png", dpi=300)
    plt.show()

    exit()

km = KMeans(n_clusters=args.k_value, max_iter=200, n_init=10)
km = km.fit(text_features)
predictedCluster_text_features = km.predict(text_features)


umap_text_features2D = umap.UMAP(n_neighbors=15,
                                 n_components=2,
                                 min_dist=0.0, metric='cosine').fit_transform(text_features)

# save umaped vectors:
df = pd.read_pickle(args.df_cases_file)
df['umapX_KMeans'] = umap_text_features2D[:, 0]
df['umapY_KMeans'] = umap_text_features2D[:, 1]
df['label_KMeans'] = predictedCluster_text_features
df.to_pickle(args.df_cases_file)

clusters = km.labels_.tolist()
docs = {'text': diag_lst, 'cluster': clusters}

# %% generate topic words with GT:
if not text_is_vectorized:
    frame = pd.DataFrame(docs, index=[clusters])
    clusters = []
    word_list_GT = []
    n_words = 10
    for cluster in range(0, args.k_value):
        t_frame = frame[frame['cluster'] == cluster]
        all_text = " ".join(t_frame['text'].astype(str))
        top_words = get_most_frequent_words(all_text, n_words)

        clusters.append(cluster)
        word_list_GT.append(top_words)
    for i in range(0, len(word_list_GT)):
        t_token = np.array(word_list_GT[i])

        if len(t_token) < n_words + 1:
            t_token = np.append(t_token, np.repeat(np.nan, n_words - len(t_token)))

        t_token = t_token.reshape((1, -1))
        if i == 0:
            token_list = t_token
        else:
            token_list = np.concatenate((token_list, t_token), axis=0)

    pd.DataFrame(token_list).to_excel('TextClustering/tables/WordsPerCluster_kmeans.xlsx',
                                      sheet_name="GT for kmeans")


# %% evalute the model by clustering metrics
from TextClustering.utils_metrics import ClusterMetrics

evaluation = ClusterMetrics(text_features, km.labels_.tolist(),
                            file_name="TextClustering/cluster_metrics/KMeans_metrics.pkl")

evaluation.write_to_file()
