# -*- coding: iso-8859-1 -*-
import pickle
from os import listdir
from os.path import isfile, join

import numpy as np
import pandas as pd
import database_preparation.utils_labeled_datasets as dt
from TextClassification.classification_for_cluster_evaluation import cross_validate_with_simple_SVM
from CorpusHomogeneity.cluster_entropy import cluster_entropy
from CorpusHomogeneity.text_entropy import corpus_entropy

recalc_cls_accuracy = True
use_always_bow_data_for_svm_accuracy = True
sort_table_by = ['s-score'] # s-score or cls accuracy
table_save_path = 'TextClustering/tables/cluster_metrics_overview'
path2corpus_bow_preprocessed = 'database/bow_prepro_diag.pkl'
path2corpus_embedding_preprocessed = 'database/embedding_prepro_diag.pkl'
scorepath = "TextClustering/cluster_metrics/"
df_cases_file = './database/df_cases.pkl'

def main():

    # ########## print cluster scores as latex table: ##################

    methodnames = ['KMeans', 'LDA', 'HDBSCAN', 'German_BERT', 'Patho_BERT', 'top2vec', 'GSDPMM']
    skipped_methods = []

    print(dt.get_all_label_set_ids())

    s_scores = []
    entropy_scores = []
    cls_ac_scores = []
    cluster_nums = []
    report_nums = []
    round_to = 3


    for label_set in methodnames:

        try:
            scores = pd.read_pickle(scorepath + label_set + "_metrics.pkl")[label_set+'_metrics']
        except:
            print(f"skipping {label_set}.")
            skipped_methods.append(label_set)
            continue
        if label_set in ['German_BERT', 'Patho_BERT', 'top2vec']:
            text_corpus_path = path2corpus_embedding_preprocessed
        else:
            text_corpus_path = path2corpus_bow_preprocessed

        try:
            s_scores.append(round(scores['s-score'],3))
        except:
            s_scores.append(None)

        try:
            cluster_nums.append(str(dt.get_amount_unique_labels(label_set)))
        except:
            cluster_nums.append(None)

        try:
            report_nums.append(str(dt.get_amount_reports(label_set)))
        except:
            report_nums.append(None)

        ### cls accuracy with svm ###
        if recalc_cls_accuracy:
            if use_always_bow_data_for_svm_accuracy:
                metrics = cross_validate_with_simple_SVM(label_set,
                                                          path2corpus_bow_preprocessed,
                                                          df_cases_file)
            else:
                metrics = cross_validate_with_simple_SVM(label_set,
                                                         text_corpus_path,
                                                         df_cases_file)

            print("================ f1-per cluster for cluster-set: " + label_set + " ================")
            df = metrics.classes_scores(-1)
            print(df.to_latex().replace('{}', 'cluster'))
            cls_ac_scores.append(round(np.mean(metrics.scores['accuracy']), round_to))
        else:
            try:
                cls_ac_scores.append(round(scores['svm-accuracy'], round_to))
            except:
                report_nums.append(None)

        ### calculate entropy ###
        with open(text_corpus_path, 'rb') as f:
            text = pickle.load(f)
        df = pd.read_pickle(df_cases_file)
        clusters = df['label_'+label_set].tolist()
        frame = pd.DataFrame({'text': text, 'cluster': clusters}, index=[clusters])
        ent = cluster_entropy(frame)
        ent_mean, ent_std = corpus_entropy(text)
        entropy_scores.append(round(ent[0] / ent_mean, round_to))

    for methodname in skipped_methods:
        methodnames.remove(methodname)

    methodnames = [n.replace("_metrics", "") for n in methodnames]
    df = pd.DataFrame({'cluster method': methodnames, 's-score': s_scores,
                       'cls accuracy': cls_ac_scores, 'rel entropy': entropy_scores,
                       'clusters': cluster_nums, 'corpus size': report_nums})
    df.sort_values(by=sort_table_by, inplace=True, ascending=False)

    latex_table = df.to_latex(index=False)
    print("%================== clustering metric scores =================")
    print(latex_table)
    print("%===================================\n\n")

    with open(table_save_path+'_latex.txt', 'w') as f:
        f.write(latex_table)

    df.to_excel(table_save_path+'.xlsx')



if __name__ == '__main__':
    main()