# -*- coding: iso-8859-1 -*-
import sys, os
import pandas as pd
from database_preparation.utils_stringpreparation import read_german_text
import argparse


def amount_names(text):
    return len(text.split('Dr.'))-1

def cut_off_by_keywords(text, keywords=['Nachtragsbefund','Nachbericht']):
    '''
    cuts of (removes) the text-part which begins with any of the passed keyword(s)
    and returns the new (shortened) text.
    '''
    for keyword in keywords:
        if keyword in text:
            text = text[:text.index(keyword)]
    return text

def get_names(text):
    names = []
    for parts in text.split('Dr. med.')[1:]:
        tokens = parts.split(' ')
        for token in tokens:
            if '.' in token:
                continue
            if True in [c.isdigit() for c in token]:
                continue
            if 'Tel' in token:
                continue
            if token in '- war Befundverwendung f�r wissenschaftliche Zwecke oder Gutachten nur mit Genehmigung des Befunders OA PD':
                continue
            names.append(token)
    return names

def add_author_labels_to_df_cases(path_to_end_sections, authors, df_cases_file = "database/df_cases.pkl"):
    df = pd.read_pickle(df_cases_file)
    filenames = df["end_text_files"]
    author_labels = []

    print(f"\nLabeling df_cases file with authors. Searching for {authors} in {path_to_end_sections}")

    for idx, filename in enumerate(filenames):

        text = cut_off_by_keywords(read_german_text(path_to_end_sections + '/' + filename))

        # detect authors in text
        authors_in_text = [0 for a in range(len(authors))]
        for j, author in enumerate(authors):
            if author in text:
                authors_in_text[j] = 1

        # if only one author detected:
        autor_combination_as_decimal = sum([pow(2, i) * n for i, n in enumerate(authors_in_text)])
        if sum(authors_in_text) == 1:
            label = authors_in_text.index(1)
        else:
            label = -1
        author_labels.append(label)

    df['label_author'] = author_labels
    df.to_pickle(df_cases_file)

    print("=> finished. Results:")
    for i, author in enumerate(authors):
        num = 0
        for label in author_labels:
            if label == i:
                num += 1
        print(author + " accured " + str(num) + " times")
    sum_no_author = 0
    for label in author_labels:
        if label == -1:
            sum_no_author = sum_no_author + 1
    print(str(sum_no_author) + " unknown authors.")

    return True

def main():
    # parse arguments:
    sys.path.append(os.getcwd())
    parser = argparse.ArgumentParser()
    parser.add_argument("--path_to_end_sections",
                        default='../DataNephroTexts/end')
    parser.add_argument("--author_names",
                        default="Name1 Name2")
    args = parser.parse_args()
    authors = args.author_names.split(' ')

    add_author_labels_to_df_cases(args.path_to_end_sections, authors)

if __name__ == '__main__':
    main()