import pandas as pd
import pickle
import numpy as np
import nltk
from sklearn.feature_extraction.text import TfidfVectorizer
from database_preparation.preprocess import print_meta_data, prepro_params_2_string

# parameters:
df_cases_file = "database/df_cases.pkl"

text_corpus_paths = ['database/embedding_prepro_diag.pkl',
                     'database/bow_prepro_diag.pkl',
                     'database/embedding_prepro_desc.pkl',
                     'database/bow_prepro_desc.pkl']
vector_corpus_paths = ['database/diagnosis_texts_vectorized_DR_preprocessed.pkl',
                       'database/diagnosis_texts_vectorized_bow_preprocessed.pkl',
                       'database/description_texts_vectorized_DR_preprocessed.pkl',
                       'database/description_texts_vectorized_bow_preprocessed.pkl']


####### functions ##########
def identity(words):
    return words


def get_trained_tfidf(texts):
    vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
    return vec.fit_transform(texts)


def save_vectorized_text(text_corpus_path, vector_corpus_path):
    with open(text_corpus_path, 'rb') as f:
        text_lst = pickle.load(f)

    text1 = np.asarray(text_lst[0])
    text_lst_is_tokenized = bool(text1.ndim)
    if not text_lst_is_tokenized:
        tokenized_texts = []
        for t_text in text_lst:
            tokenized_texts.append(nltk.tokenize.word_tokenize(t_text, language='german'))
        text_lst = tokenized_texts
        del tokenized_texts

    vectorized_text = get_trained_tfidf(text_lst)
    with open(vector_corpus_path, 'wb') as f:
        pickle.dump(vectorized_text, f)
    print(f"saved {vector_corpus_path}")

    # save metadata:
    '''with open(text_corpus_path.replace('.pkl', '_meta.pkl'), 'rb') as f:
        params = pickle.load(f)
    metadata_text = prepro_params_2_string(params)
    with open(vector_corpus_path.replace('.pkl', '_meta.txt'), 'w') as f:
        f.write(metadata_text)'''


def main():

    for i, text_corpus_path in enumerate(text_corpus_paths):
        save_vectorized_text(text_corpus_path, vector_corpus_paths[i])


if __name__ == '__main__':
    main()
